% File written by E. Smith.
% Design to take in data from sinusoidal excitation of a motor system -.csv
% Utilizes the data to investigate the amplitude and phase for the
% input and output of the system through use of the Fast Fourier Transform.
% This could be used in a loop to obtain all Frequency data and then
% generate a Bode plot and compare with theoretical motor response that
% ignores inductance.
clear all; close all; clc;

% Decide to show plots for each file?
plotdata = 0; % 0 Do not show, 1 show.
% Decide to show amplitude and phase results for each file?
ampphasedata = 0; % 0 Do not show, 1 show.
% Open File and create matrix of data, no header in file.
[filename, pathname, filterindex] = uigetfile('*.csv','MultiSelect','on');
for counter = 1:length(filename)
    M = csvread(char(filename(counter))); % Convert cell to string
    % Grab Section of Data of Interest
    [pks,locs]= findpeaks(M(:,3));
    usepk = locs(3);% use 3rd peak - prevents startup magnitude issues.
    timevec = M(usepk:length(M),1);
    outvec = M(usepk:length(M),2);
    invec = M(usepk:length(M),3);
    % Perform FFT to obtain Amplitudes and Phase Data
    dt = timevec(2)-timevec(1); % Time spacing (dt)
    Fs = 1/dt; % Sampling Rate
    N = length(timevec); % Data Length
    df = Fs/N; % Frequency spacing
    outvecfft = (2/N)*fft(outvec); % FFT Output - proper scale
    invecfft = (2/N)*fft(invec); % FFT Input - proper scale
    outvecfft(1) = 0; % Remove Offsets - DC Component
    invecfft(1) = 0;  % Remove Offsets - DC Component
    freqvec = [0:df:Fs-df]; % Full frequency spectrum
    % Plot the data?
    % Time Data
    if plotdata == 1;
        figure
        subplot(2,1,1)
        plot(M(:,1),M(:,2),M(:,1),M(:,3),'LineWidth',1.5);
        xlabel('Time (s)'); ylabel('Rev/s or Volts');title('Comparison of input and output');
        legend('Output (rev/s)','Input (Volts)')
        subplot(2,1,2)
        plot(timevec,outvec,timevec,invec,'LineWidth',1.5);
        xlabel('Time (s)'); ylabel('Rev/s or Volts');title('Truncated Comparison of input and output');
        legend('Output (rev/s)','Input (Volts)')
        % Frequency Data
        figure
        subplot(2,1,1)
        plot(freqvec,abs(outvecfft));
        ylabel('Output FFT Magnitude');title('FFT plots');
        subplot(2,1,2)
        plot(freqvec,abs(invecfft));
        ylabel('Input FFT Magnitude');xlabel('Frequency in Hz');
    end
    % Find the Amplitude
    inmax(counter) = max(abs(invecfft(1:round((length(invecfft)-1)/2))));
    outmax(counter) = max(abs(outvecfft(1:round((length(outvecfft)-1)/2))));
    inmaxloc = find(abs(invecfft)==inmax(counter));
    outmaxloc = find(abs(outvecfft)==outmax(counter));
    freq(counter) = 2*pi*freqvec(inmaxloc(1)); % FFT symmetry, so 2 results. - rad/s
    % Calculate the Phase
    thetain = atan(real(invecfft(inmaxloc(1)))/imag(invecfft(inmaxloc(1))));
    thetaout = atan(real(outvecfft(outmaxloc(1)))/imag(outvecfft(outmaxloc(1))));
    thetadiffrad = thetain-thetaout;
    thetadiff(counter) = thetadiffrad(1)/pi*180; % Convert to Degrees
    % Display Result in Matlab Interface
    if ampphasedata == 1;
        display(['Input Magnitude=',num2str(inmax)]);
        display(['Output Magnitude=',num2str(outmax)]);
        display(['Phase Difference=',num2str(thetadiff(1,1))]);
    end
    clearvars -except filename ampphasedata plotdata inmax outmax freq thetadiff
end
% Create Bode Theory and Compare Experimental Results First Order
% Assumption - Ignore Inductance -> Km/(Tm*s+1);
% 20*log10(Output/Input) - Exp.
magE = 20*log10(outmax./inmax);
phaseE = thetadiff;
% Theory:
% Km = lim as freq ->0 so B/A first value
% Tm is based off 3dB or 45 degree point
Km = outmax(1)/inmax(1);
% Calculate Tm using linear interpolation of phase.
lb = find(abs(thetadiff) < 45,1,'last'); % location
ub = find(abs(thetadiff) > 45,1,'first'); % location
thetalb = thetadiff(lb); % Theta value
thetaub = thetadiff(ub); % Theta value
freq45 = freq(lb)+(-45-thetalb)*(freq(ub)-freq(lb))/(thetaub-thetalb); % Linear Interpolation
Tm = 1/freq45;
sys = tf([Km],[Tm 1]);
% Create Bode Plot
w = [0:0.1:freq(end)*1.1];
[magB,phaseB] = bode(sys,w);
magB = 20*log10(squeeze(magB));phaseB = squeeze(phaseB); % Clean up result
% Make Plots for Theory and Experiment
figure
subplot(2,1,1)
semilogx(w,magB,'-k',freq,magE,'ob');xlabel('Frequency (rad/s)');ylabel('Out over In (dB)');
legend('Theory','Exp','Location','Best')
subplot(2,1,2)
semilogx(w,phaseB,'-k',freq,phaseE,'ob');xlabel('Frequency (rad/s)');ylabel('Phase (deg)');
legend('Theory','Exp','Location','Best')